(function()
{
   "use strict";

   var threeObj = window.THREE,

       mouseCoords = {
          xCoord: 0,
          yCoord: 0
       },

       canvasSize = {
         WIDTH: window.innerWidth,
         HEIGHT: window.outerHeight
       },

       cameraAttr = {
         VIEW_ANGLE: 90,
         ASPECT: (canvasSize.WIDTH / canvasSize.HEIGHT),
         NEAR: 0.01,
         FAR: 99999999999
       },

       INTERSECTED,
       projector = new threeObj.Projector(),
       canvasArea = document.getElementById("area"),
       renderer = new threeObj.WebGLRenderer({antialias: true, canvas: canvasArea}),
       camera = new threeObj.PerspectiveCamera(cameraAttr.VIEW_ANGLE, cameraAttr.ASPECT, cameraAttr.NEAR, cameraAttr.FAR),
       scene = new threeObj.Scene(),

       controllObj = {
         rotateSpeed: 2,
         zoomSpeed: 5,
         panSpeed: 2,
         noZoom: false,
         noPan: false,
         staticMoving: true,
         dynamicDampingFactor: 0.3,
         width: canvasSize.WIDTH,
         height: canvasSize.HEIGHT
         // controllerKeys: [65, 83, 68] // [ rotateKey, zoomKey, panKey ]
       };

   renderer.shadowMapEnabled = true;
   renderer.shadowMapSoft = true;
   renderer.setSize(canvasSize.WIDTH, canvasSize.HEIGHT);

   camera.position.x = 0;
   camera.position.y = 1000;
   camera.position.z = 600;
   // camera.lookAt(new threeObj.Vector3(100, 300, 0));


   var trackBallController = {
      trackBallObj: new threeObj.TrackballControls(camera, renderer.domElement),
      createController: function(controllObj)
      {
         this.trackBallObj.rotateSpeed = controllObj.rotateSpeed;
         this.trackBallObj.zoomSpeed = controllObj.zoomSpeed;
         this.trackBallObj.panSpeed = controllObj.panSpeed;
         this.trackBallObj.noZoom = controllObj.noZoom;
         this.trackBallObj.noPan = controllObj.noPan;
         this.trackBallObj.staticMoving = controllObj.staticMoving;
         this.trackBallObj.dynamicDampingFactor = controllObj.dynamicDampingFactor;
         this.trackBallObj.screen.width = controllObj.width;
         this.trackBallObj.screen.height = controllObj.height;
         //this.trackBallObj.keys = controllObj.controllerKeys;
//      controller.minDistance = 0;
//      controller.maxDistance = 30000;
      },
      controllerUpdate: function()
      {
         this.trackBallObj.update();
      },
      rotateCameraControl: function()
      {
         this.trackBallObj.rotateCamera();
      }
   };

   trackBallController.createController(controllObj);

   var setLight = function(lightObj, meshObject)
   {
      var spotLight = new threeObj.SpotLight(lightObj.lightColor);
      spotLight.castShadow = true;
      spotLight.shadowDarkness = 1;
      spotLight.shadowCameraVisible = lightObj.showShadowCamera;
      spotLight.position = (meshObject === null) ? new threeObj.Vector3D(lightObj.xCoord, lightObj.yCoord, lightObj.zCoord) : meshObject.position;

      //spotLight.shadowCameraRight = 5;
      //spotLight.shadowCameraLeft = -50;
      //spotLight.shadowCameraTop = 5000;
      //spotLight.shadowCameraBottom = -500;

      scene.add(spotLight);
   };

   var mesh = function(){return this;};

   mesh.lightObject = function(lightObj)
   {
      var sphereMesh = new threeObj.Mesh(new threeObj.SphereGeometry(10, 0, 0),
                                         new threeObj.MeshBasicMaterial({ color: 0xffffff, wireframe: true }));
      sphereMesh.position.x = lightObj.xCoord;
      sphereMesh.position.y = lightObj.yCoord;
      sphereMesh.position.z = lightObj.zCoord;

      scene.add(sphereMesh);

      setLight(lightObj, sphereMesh);
   };

   mesh.text = {
      create: function(text, textCoordsObj, textAttrObj, textMaterialObj)
      {
         var text3d = new threeObj.TextGeometry(text, textAttrObj);
         text3d.computeBoundingBox();

         var textMesh = new threeObj.Mesh(text3d, textMaterialObj);
         textMesh.position.x = textCoordsObj.xCoords;
         textMesh.position.y = textCoordsObj.yCoords;
         textMesh.position.z = textCoordsObj.zCoords;

         scene.add(textMesh);
      }
   };

   mesh.line = {
      create: function(lineObj)
      {
         var lineMesh = new threeObj.Line(lineObj.lineGeometry, lineObj.materialObj);

         lineMesh.position.x = lineObj.xCoords;
         lineMesh.position.y = lineObj.yCoords;
         lineMesh.position.z = lineObj.zCoords;

         lineMesh.rotation.x = lineObj.xRotation;
         lineMesh.rotation.y = lineObj.yRotation;
         lineMesh.rotation.z = lineObj.zRotation;

         return lineMesh;
      }
   };

   mesh.coordinateSystem = {
      create: function()
      {
         var geometry = new threeObj.Geometry();
         geometry.vertices.push(new threeObj.Vector3(0, 0, 0));
         geometry.vertices.push(new threeObj.Vector3(999999999999999, 0, 0));

         var lineObj = {
            xCoords: 0,
            yCoords: 0,
            zCoords: 0,
            xRotation: 0,
            yRotation: 0,
            zRotation: 0,
            lineGeometry: geometry,
            materialObj: new threeObj.LineBasicMaterial({ color: 0xff0000 })
         };

         var line = mesh.line.create(lineObj);
         scene.add(line);


         geometry = new threeObj.Geometry();
         geometry.vertices.push(new threeObj.Vector3(0, 0, 0));
         geometry.vertices.push(new threeObj.Vector3(0, 999999999999999, 0));

         lineObj.lineGeometry = geometry;
         lineObj.materialObj = new threeObj.LineBasicMaterial({ color: 0x00ff00 });

         line = mesh.line.create(lineObj);
         scene.add(line);


         geometry = new threeObj.Geometry();
         geometry.vertices.push(new threeObj.Vector3(0, 0, 0));
         geometry.vertices.push(new threeObj.Vector3(0, 0, -999999999999999));

         lineObj.lineGeometry = geometry;
         lineObj.materialObj = new threeObj.LineBasicMaterial({ color: 0x0000ff });

         line = mesh.line.create(lineObj);
         scene.add(line);


         geometry = new threeObj.Geometry();
         geometry.vertices.push(new threeObj.Vector3(-999999999999999, 0, 0));
         geometry.vertices.push(new threeObj.Vector3(0, 0, 0));

         lineObj.lineGeometry = geometry;
         lineObj.materialObj = new threeObj.LineBasicMaterial({ color: 0xff0000, opacity: .3 });

         line = mesh.line.create(lineObj);
         scene.add(line);

         geometry = new threeObj.Geometry();
         geometry.vertices.push(new threeObj.Vector3(0, -999999999999999, 0));
         geometry.vertices.push(new threeObj.Vector3(0, 0, 0));

         lineObj.lineGeometry = geometry;
         lineObj.materialObj = new threeObj.LineBasicMaterial({ color: 0x00ff00, opacity: .3 });

         line = mesh.line.create(lineObj);
         scene.add(line);


         geometry = new threeObj.Geometry();
         geometry.vertices.push(new threeObj.Vector3(0, 0, 999999999999999));
         geometry.vertices.push(new threeObj.Vector3(0, 0, 0));

         lineObj.lineGeometry = geometry;
         lineObj.materialObj = new threeObj.LineBasicMaterial({ color: 0x0000ff, opacity: .3 });

         line = mesh.line.create(lineObj);
         scene.add(line);
      }
   };

   mesh.grid = {
      create: function(/*baseMaterialObj*/)
      {
         var size = 100000, step = 100;

         var geometry = new THREE.Geometry();
         var material = new THREE.LineBasicMaterial({ color: 0x444444, opacity: 1 });

for(var i = -size; i <= size; i += step)
{
    geometry.vertices.push(new THREE.Vector3(-size, 0, i));
    geometry.vertices.push(new THREE.Vector3(size, 0, i));

    geometry.vertices.push(new THREE.Vector3(i, 0, -size));
    geometry.vertices.push(new THREE.Vector3(i, 0, size));
}

return new THREE.Line(geometry, material, THREE.LinePieces);
//scene.add( line );


         //var geometry = new threeObj.Geometry();
         //geometry.vertices.push(new threeObj.Vector3(-500000, 0, 0));
         //geometry.vertices.push(new threeObj.Vector3(500000, 0, 0));

         //var lineObj = {
           // xCoords: 0,
         //   yCoords: 0,
          //  zCoords: 0,
         //   xRotation: 0,
          //  yRotation: 0,
         //   zRotation: 0,
          //  lineGeometry: geometry,
         //   materialObj: new threeObj.LineBasicMaterial({ color: 0xcccccc, opacity: .2 })
         //};

         //var zLine = 0, xLine = 0;

         //var lineMesh = new threeObj.Mesh(new threeObj.CubeGeometry(0, 0, 0),
           //                               new threeObj.MeshBasicMaterial({ opacity: 0 }));

         //for(var i = 0; i <= 2000; i++)
         //{
           // lineObj.xCoords = 0;
            //lineObj.yCoords = 0;
            //lineObj.zCoords = (i * 500) - 500000;

            //lineObj.yRotation = 0;
            //zLine = mesh.line.create(lineObj);

            //lineObj.xCoords = (i * 500) - 500000;
            //lineObj.yCoords = 0;
            //lineObj.zCoords = 0;

            //lineObj.yRotation = 90 * Math.PI / 180;
            //xLine = mesh.line.create(lineObj);

            //lineMesh.add(xLine);
            //lineMesh.add(zLine);
         //}

         //return lineMesh;


      }
   };

   mesh.sphere = {
      create: function(sphereAttrObj)
      {
         var sphereMesh = new threeObj.Mesh(new threeObj.SphereGeometry(sphereAttrObj.radius, sphereAttrObj.segments,
                                                                        sphereAttrObj.rings), sphereAttrObj.materialObj);
         sphereMesh.castShadow = true;
         sphereMesh.receiveShadow = false;
         sphereMesh.position = new threeObj.Vector3(sphereAttrObj.xCoord, sphereAttrObj.yCoord, sphereAttrObj.zCoord);

         scene.add(sphereMesh);
      }
   };

   mesh.cube = {
      create: function(cubeAttrObj)
      {
         var cubeMesh = new threeObj.Mesh(new threeObj.CubeGeometry(cubeAttrObj.width, cubeAttrObj.height,
                                                                    cubeAttrObj.depth, cubeAttrObj.segmentsA,
                                                                    cubeAttrObj.segmentsB, cubeAttrObj.segmentsC), cubeAttrObj.materialObj);
         cubeMesh.position = new threeObj.Vector3(cubeAttrObj.xCoord, cubeAttrObj.yCoord, cubeAttrObj.zCoord);

         cubeMesh.castShadow = true;
         cubeMesh.receiveShadow = false;
         //console.log(cubeMesh.geometry.faces);

         scene.add(cubeMesh);
      }
   };

   /* Events */
   var events = function(){return this;};

   /*
    * Events for the window.
    */
   events.windowEvents = {
      onWindowResize: function()
      {
         var width = window.innerWidth;
         var height = window.innerHeight;

         renderer.setSize(width, height);

         camera.aspect = (width / height);
         camera.updateProjectionMatrix();

         trackBallController.createController(controllObj);
      }
   };

   /*
    * Events for the mouse (over, up, click, and so on).
    */
   events.mouseEvents = {
      onMouseWheel: function(){ },
      onObjMouseOver: function(event)
      {
         event.preventDefault();

         mouseCoords.xCoord = (event.clientX / window.innerWidth) * 2 - 1;
         mouseCoords.yCoord = - (event.clientY / window.innerHeight) * 2 + 1;

         var vector = new threeObj.Vector3(mouseCoords.xCoord, mouseCoords.yCoord, 1);
         projector.unprojectVector(vector, camera);

         var ray = new threeObj.Ray(camera.position, vector.subSelf(camera.position).normalize());
         var intersects = ray.intersectScene(scene);

         if(intersects.length > 0)
         {
            if(INTERSECTED != intersects[0].object)
            {
               if(INTERSECTED)
               {
                  INTERSECTED.material.color.setHex(INTERSECTED.currentHex);
               }

               INTERSECTED = intersects[0].object;
               console.log(INTERSECTED.geometry.faces);
               INTERSECTED.currentHex = INTERSECTED.material.color.getHex();
               INTERSECTED.material.color.setHex(0xff0000);
            }
         }
         else
         {
            if(INTERSECTED)
            {
               INTERSECTED.material.color.setHex(INTERSECTED.currentHex);
            }

            INTERSECTED = null;
         }

//         if(event.button === 0)
//         {

         //}
         /*
          // Parse all the faces.
          for(var i in intersects)
          {
             intersects[ i ].face.material[ 0 ].color.setHex( Math.random() * 0xffffff | 0x80000000 );
          }
         */
      }
   };

   var render = function()
   {
      requestAnimationFrame(render);
      trackBallController.controllerUpdate();
      renderer.render(scene, camera);
   };

   window.threeObj = threeObj;
   window.mesh = mesh;
   window.setLight = setLight;
   window.render = render;
   window.events = events;
   window.scene = scene;
}());

//var sphereAttributes = {
//   rings: 100,
//   segments: 2,
//   radius: 20,
//   xCoord: 100,
//   yCoord: 100,
//   zCoord: -120,
//   materialObj: new threeObj.MeshLambertMaterial({ color: 0x00CC00, wireframe: true })
//};
//mesh.sphere.create(sphereAttributes);
//
//var cubeAttributes = {
//   width: sphereAttributes.radius * 3,
//   height: sphereAttributes.radius * 3,
//   depth: sphereAttributes.radius * 3,
//   xCoord: sphereAttributes.xCoord,
//   yCoord: sphereAttributes.yCoord,
//   zCoord: sphereAttributes.zCoord,
//   segmentsA: 0,
//   segmentsB: 0,
//   segmentsC: 0,
//   materialObj: new threeObj.MeshBasicMaterial({ color: 0xFFFFFF, wireframe: true })
//};
//mesh.cube.create(cubeAttributes);

cubeAttributes = {
   width: 2000,
   height: 20,
   depth: 2000,
   xCoord: 1000,
   yCoord: 10,
   zCoord: -1000,
   segmentsA: 0,
   segmentsB: 0,
   segmentsC: 0,
   materialObj: new threeObj.MeshLambertMaterial({color: 0xaaff22, wireframe: false, opacity: 1})
};
mesh.cube.create(cubeAttributes);
window.addEventListener("resize", events.windowEvents.onWindowResize, false);

//document.getElementById("area").addEventListener("mousemove", events.mouseEvents.onObjMouseOver, false);
//window.addEventListener('mousewheel', events.mouseEvents.onMouseWheel, false);

   //window.setLight(-300, 200, 0, 0xcf1ff4, true);
   // window.render();

//         var plane = new THREE.PlaneGeometry(8, 8, 8, 8);
//
//         console.log(plane.faceVertexUvs[0]);
//   for ( i = 0; i < plane.faceVertexUvs[ 0 ].length; i ++ ) {
//
//		uvs = plane.faceVertexUvs[ 0 ][ i ];
//
//		for ( j = 0; j < uvs.length; j ++ ) {
//
//			uvs[ j ].u *= 8;
//			uvs[ j ].v *= 8;
//
//		}
//	}
